@@ -29,10 +29,16 @@
 #define MAX_OLD_DATA    ( 8 * 1024)
 #define EXPECT_OUT_NUM  (9 + 1)
 
+#define NONBLOCK_DROP_SIZE (1 * 1024)
+
 #if (SIZE_RAW_BUF + 1024) > PASS_MAX_MSG
 #error "SIZE_RAW_BUF too large compared to PASS_MAX_MSG"
 #endif
 
+#if SIZE_RAW_BUF < MAX_OLD_DATA + 2 * NONBLOCK_DROP_SIZE
+#error "SIZE_RAW_BUF too small"
+#endif
+
 /* N.B.:
  *  - Remember to update `serv_init()' accordingly when adding new fields
  *    to the struct.
@@ -213,12 +219,39 @@ serv_msg_send(ttlv_t **msg, bool free_msg)
 }
 
 static void
-serv_hello(void)
+serv_hello(ttlv_t * msg_in)
 {
+    ttlv_t * cli_version = NULL;
+    ttlv_t * msg_out = NULL;
+    char err_msg[64];
+
     if (not_CONNECTED) {
         return;
     }
 
+    cli_version = ttlv_find_child(msg_in, TAG_VERSION);
+    if (NULL == cli_version) {
+        /* client version too old */
+        debug("client version too old");
+
+        close(g.conn.sock);
+        g.conn.sock = -1;
+        return;
+    } else if ( ! streq(VERSION_, (char *)cli_version->v_text) ) {
+        /* version mismatch */
+        debug("version mismatch");
+
+        snprintf(err_msg, sizeof(err_msg),
+                 "version mismatch (server: %s, client: %s)",
+                 VERSION_, (char *)cli_version->v_text);
+        msg_out = serv_new_error(ERROR_PROTO, err_msg);
+        serv_msg_send( & msg_out, true);
+
+        close(g.conn.sock);
+        g.conn.sock = -1;
+        return;
+    }
+
     debug("sending HELLO");
     if (msg_hello(g.conn.sock) < 0) {
         debug("msg_hello failed (client dead?)");
@@ -248,6 +281,7 @@ serv_disconn(void)
     Clock_gettime( & g.lastactive);
 }
 
+static void buf_raw2expect(void);
 static void
 serv_process_msg(void)
 {
@@ -271,7 +305,7 @@ serv_process_msg(void)
 
     case TAG_HELLO:
         debug("received HELLO");
-        serv_hello();
+        serv_hello(msg_in);
 
         break;
 
@@ -440,6 +474,11 @@ serv_process_msg(void)
 
     case TAG_INFO:
         {
+            int n_expbuf = 0;
+
+            buf_raw2expect();
+            n_expbuf = MIN(g.expcnt, MAX_EXPBUF_PEEK);
+
             msg_out = ttlv_new_struct(TAG_INFO);
 
             ttlv_append_child(
@@ -453,6 +492,7 @@ serv_process_msg(void)
                 ttlv_new_int(TAG_TTL,         g.cmdopts->spawn.ttl),
                 ttlv_new_int(TAG_IDLETIME,    g.cmdopts->spawn.idle),
                 ttlv_new_int(TAG_ZOMBIE_TTL,  g.cmdopts->spawn.zombie_idle),
+                ttlv_new_raw(TAG_EXPBUF,      n_expbuf, g.expbuf + g.expcnt - n_expbuf),
                 NULL);
             serv_msg_send( & msg_out, true);
 
@@ -981,6 +1021,16 @@ serv_loop(void)
                 if (g.fd_ptm > fd_max) {
                     fd_max = g.fd_ptm;
                 }
+
+                /* [>] For non-blocking mode, we'll drop old data as necessary
+                 *     so `rawbuf' would always have free space for new output
+                 *     from pts side.
+                 */
+            } else if (spawn->nonblock) {
+                FD_SET(g.fd_ptm, & readfds);
+                if (g.fd_ptm > fd_max) {
+                    fd_max = g.fd_ptm;
+                }
             }
         }
 
@@ -1038,21 +1088,19 @@ serv_loop(void)
                 serv_read_ptm();
             }
         }
-        /* -nonblock is ON */
+        /* -nonblock is ON, drop some data when rawbuf is full */
         if (spawn->nonblock && not_CONNECTED) {
-            /* mark "new" data as "old" when -nonblock is ON */
-            if (g.newcnt > 0) {
-                g.rawnew += g.newcnt;
-                g.newcnt = 0;
-            }
+            int oldcnt;
+            oldcnt = g.rawnew - g.rawbuf;
+            if (oldcnt + g.newcnt == g.rawbufsize && oldcnt <= MAX_OLD_DATA) {
+                debug("non-blocking: rawbuf full, drop %d bytes", NONBLOCK_DROP_SIZE);
 
-            /* DO NOT UNCOMMENT THIS! */
-#if 0
-            /* mark all data as expect'ed */
-            g.expoffset = g.ntotal;
-            g.expcnt = 0;
-            g.expbuf[0] = '\0';
-#endif
+                /* Here we only mark more data as _old_ and the oldest data will
+                 * be automatically dropped in `drop_old_data()'.
+                 */
+                g.rawnew += NONBLOCK_DROP_SIZE;
+                g.newcnt -= NONBLOCK_DROP_SIZE;
+            }
         }
 
         /* new message from client */
